﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Hive.Contracts.BusinessObjects;
using HeuristicLab.Hive.Server.DataAccess;

namespace HeuristicLab.Hive.Server.LINQDataAccess {
  public class UptimeCalendarDao: BaseDao<AppointmentDto, UptimeCalendar>, IUptimeCalendarDao {
    public override UptimeCalendar DtoToEntity(AppointmentDto source, UptimeCalendar target) {
      if(source == null)
        return null;
      if (target == null)
        target = new UptimeCalendar();

      target.AllDayEvent = source.AllDayEvent;
      target.EndDate = source.EndDate;
      target.StartDate = source.StartDate;
      target.Recurring = source.Recurring;
      target.RecurringId = source.RecurringId;

      target.ResourceId = source.ResourceId;

      return target;
      
    }

    public override AppointmentDto EntityToDto(UptimeCalendar source, AppointmentDto target) {
      if (source == null)
        return null;
      if (target == null)
        target = new AppointmentDto();
      
      target.AllDayEvent = source.AllDayEvent;
      target.EndDate = source.EndDate;
      target.StartDate = source.StartDate;
      target.Recurring = source.Recurring;
      target.RecurringId = source.RecurringId;

      target.ResourceId = source.ResourceId;

      return target;

    }

    #region IGenericDao<AppointmentDto> Members

    public AppointmentDto FindById(Guid id) {
      return (from app in Context.UptimeCalendars
              where app.UptimeCalendarId.Equals(id)
              select EntityToDto(app, null)).SingleOrDefault();
    }

    public IEnumerable<AppointmentDto> FindAll() {
      return (from app in Context.UptimeCalendars              
              select EntityToDto(app, null)).ToList();
    }

    public AppointmentDto Insert(AppointmentDto bObj) {
      UptimeCalendar uc = DtoToEntity(bObj, null);
      Context.UptimeCalendars.InsertOnSubmit(uc);
      CommitChanges();
      bObj.Id = uc.UptimeCalendarId;
      return bObj;
    }

    public void Delete(AppointmentDto bObj) {
      Context.UptimeCalendars.DeleteOnSubmit(Context.UptimeCalendars.SingleOrDefault(uc => uc.UptimeCalendarId.Equals(bObj.Id)));
      CommitChanges();
    }

    public void Update(AppointmentDto bObj) {
      UptimeCalendar cc = Context.UptimeCalendars.SingleOrDefault(c => c.UptimeCalendarId.Equals(bObj.Id));
      DtoToEntity(bObj, cc);
      CommitChanges();
    }

    public IEnumerable<AppointmentDto> GetUptimeCalendarForResource(Guid resourceId) {
      return (from uc in Context.UptimeCalendars
              where uc.ResourceId.Equals(resourceId)
              select EntityToDto(uc, null)).ToList();
    }

    public void SetUptimeCalendarForResource(Guid resourceId, IEnumerable<AppointmentDto> appointments) {
      var q = (from uc in Context.UptimeCalendars
               where uc.ResourceId.Equals(resourceId)
               select uc);
      
      Context.UptimeCalendars.DeleteAllOnSubmit(q);

      foreach (AppointmentDto appdto in appointments) {      
        UptimeCalendar uc = DtoToEntity(appdto, null);
        uc.ResourceId = resourceId;
        Context.UptimeCalendars.InsertOnSubmit(uc);
      }

      CommitChanges();            
    }    

    public IEnumerable<AppointmentDto> GetCalendarForClient(ClientDto client) {
      Client dbc = Context.Resources.OfType<Client>().SingleOrDefault(c => c.ResourceId.Equals(client.Id));
      IList<AppointmentDto> appointments = new List<AppointmentDto>();
      if (dbc != null) {
        ClientGroup cg =
          Context.Resources.OfType<ClientGroup>().SingleOrDefault(cgroup => cgroup.ResourceId.Equals(dbc.UseCalendarFromResourceId));
        //in case no plan has been set
        if (cg == null)
          if (dbc.ClientGroup_Resources_Parents.FirstOrDefault() != null)
            cg = dbc.ClientGroup_Resources_Parents.FirstOrDefault().ClientGroup;

        if (cg == null)
          return appointments;

        while (cg.UptimeCalendars.Count == 0) {
          if (cg.ClientGroup_Resources_Parents.FirstOrDefault() != null)
            cg = cg.ClientGroup_Resources_Parents.FirstOrDefault().ClientGroup;
          else {
            break;
          }
        }

        foreach (UptimeCalendar appointment in cg.UptimeCalendars) {
          appointments.Add(EntityToDto(appointment,null));  
        }

      }
      return appointments;
    }

    public void NotifyClientsOfNewCalendar(Guid groupId, bool forcePush) {
      
      //Get the current ClientGroup
      ClientGroup cg = Context.Resources.OfType<ClientGroup>().SingleOrDefault(cgroup => cgroup.ResourceId.Equals(groupId));
      if(cg == null)
        return;

      //Get all the affected clients
      List<Client> clients = Context.Resources.OfType<Client>().Where(c => c.UseCalendarFromResourceId.Equals(cg.ResourceId)).ToList();
      
      //Set new state
      foreach (Client client in clients) {
        client.CalendarSyncStatus = (forcePush ? Enum.GetName(typeof(CalendarState), CalendarState.ForceFetch) : Enum.GetName(typeof(CalendarState), CalendarState.Fetch));      
      }
      
      CommitChanges();

      //Get all Subgroups
      List<ClientGroup> groups = (from cg1 in Context.Resources.OfType<ClientGroup>()
                                  where cg1.ClientGroup_Resources_Parents.Any(
                                cgr => cgr.ClientGroupId.Equals(groupId))
                              select cg1).ToList();

      //If they have their own calendar - stop propagation
      //otherweise - propagate
      foreach (ClientGroup cgroup in groups) {
        if(cgroup.UptimeCalendars.Count == 0)
          NotifyClientsOfNewCalendar(cgroup.ResourceId, forcePush);        
      }
    }

    #endregion
  }
}
